
clear;
close all;

addpath(genpath(pwd));
%% Maros Demo
fname{1} = 'AUG2DCQP';
fname{2} = 'CONT-101';
fname{3} = 'CONT-201';
fname{4} = 'CVXQP3_L';
fname{5} = 'LISWET5';
fname{6} = 'LISWET11';
fname{7} = 'POWELL20';
fname{8} = 'Q25FV47';
fname{9} = 'QPILOTNO';
fname{10} = 'QSHIP12L';
fname{11} = 'STADAT1';
fname{12} = 'UBH1'; 
idxMaros = 1:12;
%% QAP 
fname{13} = 'lipa50a';
fname{14} = 'sko56';
fname{15} = 'tai50a';
fname{16} = 'wil50';
fname{17} = 'esc64a';
idxQAP = 13:17;
%% Quartic 
fname{18} = 'be100.1';
fname{19} = 'bqp100-1';
fname{20} = 'gka5b';
idxQuartic = 18:20;
%% Portfolio 
fname{21} = 'Portfolio5';
fname{22} = 'Portfolio8';
idxPortfolio = 21:22;

idx = 1:22;

dataDir = [pwd, filesep, 'dataDemo', filesep];

for k = 1
    probname = fname{k};
    %% load data
    load([dataDir, probname, '.mat']);
    %% transform problem into standard QP format
    mE = length(bE);
    mI = length(bI);
    n = length(c);
    if isstruct(Q)
        Qmat = Q.Qmat;
    else
        Qmat = Q;
    end
    [Qinput, AA, bb, cc, LL, UU] = standardQP(Qmat, c, AE, AI, bE, bI, L, U);
    QQ.Qmat = Qinput;
    if ismember(k, idxQAP)
        QQ.Qmap = @(x) QAPmap(x, Q.QAPdata.A, Q.QAPdata.B, Q.QAPdata.S, Q.QAPdata.T);
    elseif ismember(k, idxQuartic)
        Qmap = Q.Qmap;
        QQ.Qmap = @(x) [Qmap(x(1:n)); zeros(mI, 1)];
    end
    %% options
    clear options
    options.stopTol = 1e-6;
    options.maxIterPhaseOne = 1000;
    options.maxIterPhaseTwo = 1000;
    options.sigma = 1e0;
    options.iterComputeQx = 1;
    options.useSGS = 1;
    if ismember(k, idxQAP)
        options.AATMethod = 'direct';
        options.QMethod   = 'inexact';
        options.SSNMethod = 'bicgstab';
        options.kappa = 1e0;
        options.sigmaPhaseTwo = 1e2;
        options.useSGS = 0;
    elseif ismember(k, idxQuartic)
        options.sigma = 5e-1;
        options.kappa = 5e1;
        options.AATMethod = 'inexact';
        options.QMethod   = 'inexact';
        options.SSNMethod = 'bicgstab';
    elseif ismember(k, idxPortfolio)
        options.AATMethod = 'direct';
        options.QMethod   = 'inexact';
        options.SSNMethod = 'chol';
        options.maxIterPhaseOne = 10;
        options.sigmaPhaseTwo = 1e3;
        options.kappa = 1e-4;
        options.isDiagonalQ = 0;
        options.iterComputeQx = 20;
    else
        options.AATMethod = 'direct';
        options.QMethod   = 'direct';
        options.SSNMethod = 'chol';
        options.kappa = 1e-4;
        if k == 1; options.sigma = 1e-2; end
    end
    %% Solve QP
    res = QPPAL_main(QQ, AA, bb, cc, LL, UU, options);
end