%%
%% Perform Ruiz equilibration for scaling a m by n matrix
%% Ling Liang, 22-12-2020.
%%
function [E, D, Anew] = RuizEquilibration(A, maxit)
[m,n] = size(A);
Anew = A; D = ones(n, 1); E = ones(m, 1);
if (nargin < 2); maxit = 10; end
option = 1;
tol = 1e-6;
for it = 1:maxit
    if (option == 1)
        Etmp = min(1e2, max(1e-2, full(sqrt(max(abs(Anew), [], 2)))));
        Dtmp = min(1e2, max(1e-2, full(sqrt(max(abs(Anew), [], 1)'))));
    elseif (option == 2)
        Etmp = min(1e2, max(1e-2, full(sqrt(sum(Anew.*Anew, 2)))));
        Dtmp = min(1e2, max(1e-2, full(sqrt(sum(Anew.*Anew, 1)')))); 
    end
    invEtmp = 1 ./ Etmp; invDtmp = 1 ./ Dtmp;
    Anew = spdiags(invEtmp, 0, m, m) * (Anew * spdiags(invDtmp, 0, n, n));
    D = D .* invDtmp; E = E .* invEtmp;
    err = max(norm(1 - Etmp.^2), norm(1 - Dtmp.^2));
    if (err < tol)
        %fprintf('\n Ruiz Equilibration converges with err = %2.1e, tol = %2.1e, it = %2.0f', ...
        %    err, tol, it);
        break;
    end
end
end