
%%*************************************************************************
%% mybicgstab.m
%% the given tol is relative tolerance
%% tol = const* norm(b);
%%*************************************************************************

function [xx,resnrm,iter,matveciter,flag] = mybicgstab(matvecfname, prob, b, par)

N = length(b);
if ~isfield(par, 'maxit'); maxit = max(100,sqrt(N)); else; maxit = par.maxit; end
if ~isfield(par, 'tol'); tol = 1e-6*norm(b); else; tol = par.tol; end
resnrm = zeros(maxit + 1, 1);
printlevel = 0;
flag = 1;
x = zeros(N,1);
initial = 0;
if isfield(par,'initial'); initial = par.initial; end
if initial && isfield(par,'x0'); x = par.x0; end

if (norm(x)>0)
    Ax = feval(matvecfname,prob,x,par);
    r = b - Ax;
else
    r = b;
end
err = norm(r); resnrm(1) = err;  minresnrm = err; xx = x;

omega  = 1.0;
r_tld = r;
matveciter = 0;  % number of matrix-vector multiplication AX
if err < tol; iter = 0; return; end
%%
%%
breakyes = 0;
smtol = 1e-30;
for iter = 1:maxit
    rho   = (r_tld'*r);
    if (abs(rho) < smtol)
        flag = 2;
        if (printlevel > 0); fprintf('bicgstab# '); end
        breakyes = 1;
        break;
    end
    if (iter > 1)
        beta  = (rho/rho_1)* (alp/omega);
        p = r + beta*(p - omega*v);
    else
        p = r;
    end
    p_hat = precondfun(par,p);
    v = feval(matvecfname,prob,p_hat,par);
    alp = rho / (r_tld'*v);
    s = r - alp*v;
    matveciter = matveciter +1;
    
    % % comment on Aug-06-2010. Then Bicgstab would perform better
    % % have a check of this part later !
    %      if (norm(s)<tol)
    %          flag = 0;
    %          xx = x+ alp*p_hat;
    %          resnrm(iter+1) = norm(s);
    %          if (printlevel); fprintf('**'); end
    %          break;
    %      end
    
    %%
    s_hat = precondfun(par,s);
    t = feval(matvecfname,prob,s_hat,par);
    omega = (t'*s) / (t'*t);
    x = x + alp*p_hat + omega*s_hat;
    r = s - omega*t;
    rho_1 = rho;
    matveciter = matveciter +1;
    %%
    %% check convergence
    %%
    err = norm(r); resnrm(iter+1) = err;
    if (err < minresnrm)
        xx = x; minresnrm = err;
    end
    if (err < tol)
        break;
    end
    
    %      % if (err > 10*minresnrm)
    %      if (err > 100*minresnrm)
    %         if (printlevel); fprintf('^'); end
    %         breakyes = 2;
    %         break;
    %      end
    
    if (abs(omega) < smtol)
        flag = 2;
        if (printlevel > 0); fprintf('bicgstab* '); end
        breakyes = 1;
        break;
    end
end
if (~breakyes) && (printlevel >=3); fprintf('bicgstab^^'); end
resnrm = resnrm(1:1+iter);
end
%%

%%*************************************************************************
%% precondfun:
%%*************************************************************************
function  q = precondfun(par,r)
    precond = 0;
    if isfield(par,'precond'); precond = par.precond; end

    if (precond == 0)
        q = r;
    elseif (precond == 1)
        q = par.invdiagM.*r;
    elseif (precond == 3)
        rq = r(1:par.Qx);
        ry = r(par.Qx+1:end);
        dqlast = par.Qd(end);
        tmp1 = 1/(1+0.5*dqlast*par.sig);
        tmp2 = 1./(1+par.sig*par.Qd) -tmp1;
        rq = tmp1*rq + par.QV*(tmp2.*(par.QV'*rq));
        dylast = par.yd(end);
        tmp1 = 2/(par.sig*dylast);
        tmp2 = 1./(par.sig*par.yd) - tmp1;
        ry = tmp1*ry + par.yV*(tmp2.*(par.yV'*ry));
        q = [rq;ry];
    end
end


