%%******************************************************************
%% Cholesky decompostion for A*At + lambda*I
%%******************************************************************

function [L,AAt] = mycholAAt(A,lambda)

if (nargin==1); lambda = 1e-14; end
m = size(A,1);
AAt = A*A' + lambda*speye(m);
pertdiag = 1e-15*norm(diag(AAt))*ones(m,1);
AAt = AAt + spdiags(pertdiag,0,m,m);
%%
if (nnz(AAt) < 0.3*m*m); use_spchol=1; else; use_spchol=0; end
if (use_spchol)
    [L.R,L.p,L.perm] = chol(AAt,'vector');
    L.Rt = L.R';
    L.matfct_options = 'spchol';
else
    if issparse(AAt); AAt = full(AAt); end
    L.matfct_options = 'chol';
    L.perm = 1:m;
    [L.R,indef] = chol(AAt);
    L.indef = indef;
    if (indef)
        fprintf('\n mycholAAt: Matrix not positive definite');
    end
end
dd = diag(AAt);
L.isdiagonal = false;
if (norm(AAt-spdiags(dd,0,m,m),'fro') < 1e-15*norm(dd))
    L.isdiagonal = true;
    L.diagonal = dd;
end
%%******************************************************************
