function [prob, sol, scl, QSolve, ws, param, normP, normD] = ...
        rescalingPhaseOne(prob, sol, scl, QSolve, ws, param, options)
    normP = ws.normx;
    normD = max(ws.normz, norm(ws.Rd - sol.z));
    normPDratio = max(normP/normD, normD/normP);

    if normPDratio > 1.1
        bscale2 = normP;
        cscale2 = normD;
        prob.b = prob.b / bscale2;
        prob.c = prob.c / cscale2;
        prob.L = prob.L / bscale2;
        prob.U = prob.U / bscale2;
        prob.normb = norm(prob.b);
        prob.normc = norm(prob.c);

        sol.x = sol.x / bscale2;
        sol.z = sol.z / cscale2;
        if prob.existA
            sol.y = sol.y / cscale2;
        end
        sol.w = sol.w / bscale2;
        sol.Qx = sol.Qx / cscale2;
        sol.Qw = sol.Qw / cscale2;

        scl.bscale = scl.bscale * bscale2;
        scl.cscale = scl.cscale * cscale2;
        scl.objscale = scl.objscale * bscale2 * cscale2;

        param.sigma = param.sigma * (cscale2 / bscale2);
        ws = workSpace(prob, sol, scl);

        if prob.existQ
            bcratio = bscale2 / cscale2;
            if isfield(prob.Q, 'Qmat')
                prob.Q.Qmat = prob.Q.Qmat * bcratio;
            end
            if isfield(prob.Q, 'Qmap')
                prob.Q.Qmap = @(x) prob.Q.Qmap(x) * bcratio;
            end
            if strcmp(options.QMethod, 'diagonal')
                QSolve.diagQ = QSolve.diagQ * bcratio;
            elseif strcmp(options.QMethod, 'inexact')
                parQ = QSolve.parQ;
                parQ.sig = param.sigma;
                if QSolve.parQ.precond == 2
                    parQ.dQ = parQ.dQ * bcratio;
                end
                QSolve.parQ = parQ;
            end
        end
    end
end